require "advanced_report/gruff"

class AdvancedReportController < ApplicationController

  helper :timelog
  helper :custom_fields

  def index
    load_settings
    if User.current.admin?
      @all_projects = Project.find(:all, :order => "name ASC")
    else
      @all_projects = User.current.projects.find(:all, :order => "name ASC")
    end
    @all_fields = CustomField.find(:all, :conditions => "type = 'ProjectCustomField'", :order => "name ASC")
    retrieve_date_range

    @selected_projects = params[:selected_projects].is_a?(String) ? eval(params[:selected_projects]) : params[:selected_projects]
    @selected_projects ||= []
    @selected_projects.collect! { |p| p.to_i }
    @selected_members = params[:selected_members].is_a?(String) ? eval(params[:selected_members]) : params[:selected_members]
    @selected_members ||= []
    @selected_members.collect! { |m| m.to_i }
    @selected_fields = params[:selected_fields].is_a?(String) ? eval(params[:selected_fields]) : params[:selected_fields]
    @selected_fields ||= []
    @selected_fields.collect! { |f| f.to_i }
      
    empty_days = []
    (@from .. @to).each do
      empty_days << 0.0
    end
      
    @projects = []
    @projects_empty = true
    @projects_totals = empty_days.clone
    @projects_total = 0.0
    @fields = {}
    @selected_fields.each do |field_id|
      field = CustomField.find_by_id(field_id)
      if field != nil
        @fields[field_id] = {:field => field, :projects => [], :has_values => false}
      end
    end
    @show_released_versions = false
    @all_projects.select { |p| @selected_projects.include?(p.id) }.sort.each do |project|
      issues = []
      members = {}
      project_totals = empty_days.clone
      project_total = 0.0
      versions = []
      project.time_entries.find(:all, :order => "spent_on").each do |time_entry|
        if (time_entry.spent_on >= @from and time_entry.spent_on <= @to) and
           (@selected_members.empty? or @selected_members.include?(time_entry.user_id))
          day = time_entry.spent_on - @from
          if !(time_entry.issue.nil?) and !(issues.include?(time_entry.issue))
            issues << time_entry.issue
          end

          if !(members.include?(time_entry.user_id))
            members[time_entry.user_id] = {:member => time_entry.user, :days => empty_days.clone, :total => 0.0}
          end
          members[time_entry.user_id][:days][day] += time_entry.hours
          members[time_entry.user_id][:total] += time_entry.hours
          project_totals[day] += time_entry.hours
          project_total += time_entry.hours
          @projects_totals[day] += time_entry.hours
          @projects_total += time_entry.hours
          @projects_empty = false
        end
      end
      
      project.custom_values.each do |custom_value|
        if @fields.include?(custom_value.custom_field_id)
          if !(custom_value.value.empty?)
            @fields[custom_value.custom_field_id][:projects] << {:project => project, :value => custom_value}
            @fields[custom_value.custom_field_id][:has_values] = true
          end
        end
      end
      
      versionsIncluded = {}
      (@from .. @to).each do |day|
        versionsIncluded[day] = []
      end
      project_has_versions = false
      project.versions.find(:all).sort.each do |version|
        if version.effective_date != nil && versionsIncluded.include?(version.effective_date)
          versionsIncluded[version.effective_date] << version
          @show_released_versions = true
          project_has_versions = true
        end
      end
      versions = []
      (@from .. @to).each do |day|
        versions << versionsIncluded[day]
      end
      
      @projects << {:project => project,
                    :issues => issues.sort { |a, b| a.subject.downcase <=> b.subject.downcase },
                    :members => members.values.sort { |a, b| a[:member].name.downcase <=> b[:member].name.downcase },
                    :totals => project_totals,
                    :total => project_total,
                    :versions => project_has_versions ? versions : {}}

    end

    @root_projects = User.current.projects.find(:all, :conditions => "parent_id IS NULL").sort

    @title = ""
    if !@projects.empty?
      title_projects = []
      @projects.each do |project|
        title_projects << project if project[:project].parent_id == nil
      end
      if !title_projects.empty?
        @title << "#{title_projects.collect {|p| p[:project].name}.join(", ")}: "
      end
      @title << "#{l(:label_status_report, {:from => @from, :to => @to})}"
    end

    @members = []
    @members_empty = true
    @members_totals = empty_days.clone
    @members_total = 0.0
    @selected_members.each do |member_id|
      member = User.find(member_id)    
      issues = []
      projects = {}
      member_totals = empty_days.clone
      member_total = 0.0
      TimeEntry.find(:all, :conditions => "user_id = #{member.id}", :order => "spent_on").each do |time_entry|
        if (time_entry.spent_on >= @from and time_entry.spent_on <= @to) and
           (@selected_members.empty? or @selected_members.include?(time_entry.user_id)) and
           (@selected_projects.empty? or @selected_projects.include?(time_entry.project_id))
          day = time_entry.spent_on - @from
          if !(time_entry.issue.nil?) and !(issues.include?(time_entry.issue))
            issues << time_entry.issue
          end
      
          if !(projects.include?(time_entry.project_id))
            projects[time_entry.project_id] = {:project => time_entry.project, :days => empty_days.clone, :total => 0.0}
          end
          projects[time_entry.project_id][:days][day] += time_entry.hours
          projects[time_entry.project_id][:total] += time_entry.hours
          member_totals[day] += time_entry.hours
          member_total += time_entry.hours
          @members_totals[day] += time_entry.hours
          @members_total += time_entry.hours
          @members_empty = false
        end
      end

      @members << {:member => member,
                   :issues => issues.sort { |a, b| a.subject.downcase <=> b.subject.downcase },
                   :projects => projects.values.sort { |a, b| a[:project].name.downcase <=> b[:project].name.downcase },
                   :totals => member_totals,
                   :total => member_total}
    end

    respond_to do |format|
      format.html { render }
      format.pdf  { render(:type => "application/pdf",
                           :disposition => "inline",
                           :filename => "report.pdf") }
    end
  end
  
  def update_members
    @selected_projects = params[:selected_projects]
    @selected_projects ||= []
    @selected_projects.collect! { |p| p.to_i }

    @selected_members = params[:selected_members]
    @selected_members ||= []
    @selected_members.collect! { |m| m.to_i }

    @members = []
    @selected_projects.each do |project|
      Project.find(project).users.each do |member|
        @members << member unless @members.include?(member)
      end
    end
    @members.sort!

    render :layout => false
  end
  
  def time_log_graph
    load_settings
    @projects = []
    eval(params[:projects]).each do |project_id|
      project = Project.find(:first, :conditions => "id = #{project_id}")
      if project != nil
        @projects << project
      end
    end
    @members = eval(params[:members])
    retrieve_date_range
    @width = params[:width] || 800
    @height = params[:height] || 500 
    @week_count = (params[:week_count] || 16).to_i

    g = AdvancedReport::Gruff::Line.new("#{@width}x#{@height}")
    g.hide_title = true
    g.theme = graph_theme
    g.labels_x_rotation = -15.0
    g.marker_count = 5
    g.marker_font_size = 12
    g.legend_font_size = 12

    end_date = @to
    end_date -= (end_date.cwday - 1)
    start_date = end_date - ((@week_count - 1) * 7) 
    dates = []
    @week_count.times do |week|
      dates << end_date - (week * 7)
    end
    end_date += 6
    dates = dates.reverse
  
    time_entries = find_time_entries(@projects, @projects, start_date, end_date)
    users = []
    hours = []
    time_entries.each do |time_entry|
      if (@members.empty? or @members.include?(time_entry.user.id)) and 
         users.index(time_entry.user.id).nil?
        users << time_entry.user.id
        hours << [time_entry.user.name]
      end
    end
  
    @week_count.times do |week|
      hours.each do |user|
        user << 0.0
      end
    end
  
    time_entries.each do |time_entry|
      user_index = users.index(time_entry.user.id)
      if !user_index.nil?
        date = time_entry.spent_on
        index = ((date - start_date) / 7).floor
        hours[user_index][index + 1] += time_entry.hours.to_f
      end
    end
                
    dates.length.times do |i|
      g.labels[i] = dates[i].to_s
    end
  
    hours.each_index do |user|
      g.data(hours[user].shift, hours[user])
    end
    
    headers["Content-Type"] = "image/png"
    send_data(g.to_blob, :type => "image/png", :disposition => "inline")
  end

  def burn_down_graph
    load_settings
    @projects = []
    eval(params[:projects]).each do |project_id|
      project = Project.find(:first, :conditions => "id = #{project_id}")
      if project != nil
        @projects << project
      end
    end
    retrieve_date_range
    @width = params[:width] || 800
    @height = params[:height] || 400
    @week_count = (params[:week_count] || 16).to_i

    g = AdvancedReport::Gruff::Line.new("#{@width}x#{@height}")
    g.hide_title = true
    g.theme = graph_theme
    g.labels_x_rotation = -15.0
    g.marker_count = 5
    g.marker_font_size = 12
    g.hide_legend = true

    end_date = Date.today
    end_date -= (end_date.cwday - 1)
    dates = []
    @week_count.times do |week|
      dates << end_date - (week * 7)
    end
    end_date += 6
    dates = dates.reverse

    issues = find_issues(@projects, @projects)

    datas = []
    @week_count.times do |week|
      count = 0
      issues.each do |issue|
        if issue.due_date.nil? or (issue.due_date > (dates[week] + 7))
          count += 1
        end
      end
      datas << count
    end

    dates.length.times do |i|
      g.labels[i] = dates[i].to_s
    end

    g.data("None", datas)

    headers["Content-Type"] = "image/png"
    send_data(g.to_blob, :type => "image/png", :disposition => "inline")
  end

private

  def retrieve_date_range
    logger.info("retrieve_date_range()")
    @free_period = false
    @from, @to = nil, nil

    if params[:period_type] == '1' || (params[:period_type].nil? && !params[:period].nil?)
      case params[:period].to_s
      when 'today'
        @from = @to = Date.today
        logger.info("Fecha hoy: " + @from.to_s)
      when 'yesterday'
        @from = @to = Date.today - 1
      when 'current_week'
        @from = Date.today - (Date.today.cwday - 1)%7
        @to = @from + 6
      when 'last_week'
        @from = Date.today - 7 - (Date.today.cwday - 1)%7
        @to = @from + 6
      when '7_days'
        @from = Date.today - 7
        @to = Date.today
      when 'current_month'
        @from = Date.civil(Date.today.year, Date.today.month, 1)
        @to = (@from >> 1) - 1
      when 'last_month'
        @from = Date.civil(Date.today.year, Date.today.month, 1) << 1
        @to = (@from >> 1) - 1
      when '30_days'
        @from = Date.today - 30
        @to = Date.today
      when 'current_year'
        @from = Date.civil(Date.today.year, 1, 1)
        @to = Date.civil(Date.today.year, 12, 31)
      end
    elsif params[:period_type] == '2' || (params[:period_type].nil? && (!params[:from].nil? || !params[:to].nil?))
      begin; @from = params[:from].to_s.to_date unless params[:from].blank?; rescue; end
      begin; @to = params[:to].to_s.to_date unless params[:to].blank?; rescue; end
      @free_period = true
    else
      # default
    end

    @from, @to = @to, @from if @from && @to && @from > @to
    @from ||= (TimeEntry.minimum(:spent_on, :include => :project, :conditions => Project.allowed_to_condition(User.current, :view_time_entries)) || Date.today) - 1
    @to   ||= (TimeEntry.maximum(:spent_on, :include => :project, :conditions => Project.allowed_to_condition(User.current, :view_time_entries)) || Date.today)
  end

  def load_settings
    @settings = {"project_list_size" => Setting.plugin_advanced_report["project_list_size"].to_i,
                 "time_log_graph_week_count" => Setting.plugin_advanced_report["time_log_graph_week_count"].to_i,
                 "time_log_graph_width" => Setting.plugin_advanced_report["time_log_graph_width"].to_i,
                 "time_log_graph_height" => Setting.plugin_advanced_report["time_log_graph_height"].to_i,
                 "burn_down_graph_week_count" => Setting.plugin_advanced_report["burn_down_graph_week_count"].to_i,
                 "burn_down_graph_width" => Setting.plugin_advanced_report["burn_down_graph_width"].to_i,
                 "burn_down_graph_height" => Setting.plugin_advanced_report["burn_down_graph_height"].to_i}
  end
  
  def find_time_entries(all_projects, projects, start_date, end_date)
    time_entries = []
    projects.each do |project|
      time_entries += project.time_entries.find(
        :all, 
        :conditions => "spent_on >= '" + start_date.to_s + "' AND spent_on <= '" + end_date.to_s + "'", 
        :order => "hours")
      children = Project.find(
        :all, 
        :conditions => ("parent_id = " + project.id.to_s))
      children.each do |child|
        if !(all_projects.include?(child))
          child_array = []
          child_array << child
          time_entries += find_time_entries(all_projects, child_array, start_date, end_date)
        end
      end
    end
    time_entries
  end

  def find_issues(all_projects, projects)
    issues = []
    if projects != nil
      projects.each do |project|
        issues += project.issues
        Project.find(:all, :conditions => "parent_id = #{project.id.to_s}").each do |child|
          if !(all_projects.include?(child))
            issues += find_issues(all_projects, [child])
          end
        end
      end
    end
    issues
  end

  def graph_theme
    {
      :colors => ["#DB2626", "#6A6ADB", "#64D564", "#F727F7", "#EBEB20", "#303030", "#12ABAD", "#808080", "#B7580B", "#316211"],
      :marker_color => "#AAAAAA",
      :background_colors => ["#FFFFFF", "#FFFFFF"]
    }
  end

  class Helper
    class << self
      include AdvancedReportHelper
    end
  end

end
