module AdvancedReportHelper

  include Redmine::I18n
  include Redmine::Export::PDF
  include ApplicationHelper
  include ActionView::Helpers::TextHelper
  include ActionView::Helpers::UrlHelper
  include ERB::Util

  def project_tree_select(projects, 
                          list_size, 
                          selected_projects = nil, 
                          level = 0, 
                          init = true)
    html = ""
    if init
      html << "<select name=\"selected_projects[]\" id=\"selected_projects\" multiple size=\"#{list_size}\" onChange=\"#{remote_function(:update => :selected_members, :url => {:action => :update_members}, :with => "Form.Element.serialize(this)+'&'+Form.Element.serialize($('selected_members'))")};\">\n"
    end
    if projects != nil and projects.length > 0
      projects.collect do |project|
        html << "  <option value=\"#{project.id}\" style=\"padding-left:#{level * 20}px\""
        html << " selected=\"selected\"" if selected_projects != nil and selected_projects.include?(project.id)
        html << ">#{project.name}</option>\n"
        children = User.current.projects.find(:all, :conditions => "parent_id = #{project.id}").sort
        html << project_tree_select(children, 
                                    list_size, 
                                    selected_projects, 
                                    level + 1, 
                                    false)
      end
    end
    html << "</select>\n" if init
    return html
  end

  def report_to_pdf(from, to, projects, projects_totals, projects_total, members, members_totals, members_total, fields, show_released_versions)
    pdf = IFPDF.new(current_language)
    pdf.SetTitle("#{l(:advanced_report_title)} #{from} - #{to}")
    pdf.AliasNbPages
    pdf.footer_date = format_date(Date.today)
    pdf.AddPage

    write_line(pdf, "#{l(:advanced_report_title)} #{from} - #{to}", :style => "B", :font_size => 11)
    pdf.Ln

    if !(projects.empty?)
      write_line(pdf, "#{l(:label_activity_list_by_project)}:", :style => "B")
      projects.each do |project|
        if !(project[:issues].empty?)
          write_line(pdf, "- #{project[:project].name}:", :style => "I")
          project[:issues].each do |issue|
            write_line(pdf, "   - #{issue.tracker} ##{issue.id}: #{h(issue.subject)}", :style => "")
          end
        end
      end
      pdf.Ln
    end

    if !(members.empty?)
      write_line(pdf, "#{l(:label_activity_list_by_member)}:", :style => "B")
      members.each do |member|
        write_line(pdf, "- #{member[:member].name}:", :style => "I")
        member[:issues].each do |issue|
          write_line(pdf, "   - #{issue.tracker} ##{issue.id}: #{h(issue.subject)}", :style => "")
        end
      end
      pdf.Ln
    end

    fields.each_value do |field|
      if field[:has_values]
        write_line(pdf, "#{h(field[:field].name)}:", :style => "B")
        field[:projects].each do |project|
          write_line(pdf, "- #{h(project[:project].name)}:", :style => "")
          write_line(pdf, "- #{h(show_value(project[:value]))}:", :style => "")
        end
      end
      pdf.Ln
    end

    if !(projects.empty?)
      write_line(pdf, "#{l(:label_time_spent_by_project)}:", :style => "B")
      column_size = 190 / (to - from + 4)
      pdf.SetFillColor(230, 230, 230)
      pdf.SetFontStyle("", 9)
      column_options = {:width => column_size, :border => "LRTB"}
      text(pdf, l(:label_project), column_options.merge({:fill => 1}))
      text(pdf, l(:label_member), column_options.merge({:fill => 1}))
      (from .. to).each do |day|
        text(pdf, day.to_s, column_options.merge({:align => "C", :fill => 1}))
      end
      text(pdf, l(:label_total), column_options.merge({:align => "C", :fill => 1}))
      pdf.Ln

      pdf.SetFillColor(255, 255, 255)
      projects.each do |project|
        if !(project[:members].empty?)
          text(pdf, project[:project].name, column_options.merge({:width => column_size * 2, :style => "I"}))
          project[:totals].each do |total|
            text(pdf, (total > 0.0) ? ("%.2f" % total) : "", column_options.merge({:align => "C", :style => "I"}))
          end
          text(pdf, (project[:total] > 0.0) ? ("%.2f" % project[:total]) : "", column_options.merge({:align => "C", :style => "I"}))
          pdf.Ln

          project[:members].each do |member|
            text(pdf, member[:member].name, column_options.merge({:width => column_size * 2, :align => "R", :style => ""}))
            member[:days].each do |day|
              text(pdf, (day > 0.0) ? ("%.2f" % day) : "", column_options.merge({:align => "C", :style => ""}))
            end
            text(pdf, (member[:total] > 0.0) ? ("%.2f" % member[:total]) : "", column_options.merge({:align => "C", :style => ""}))
            pdf.Ln
          end
        end
      end

      text(pdf, l(:label_total), column_options.merge({:width => column_size * 2, :style => "I"}))
      projects_totals.each do |total|
        text(pdf, (total > 0.0) ? ("%.2f" % total) : "", column_options.merge({:align => "C", :style => "I"}))
      end
      text(pdf, (projects_total > 0.0) ? ("%.2f" % projects_total) : "", column_options.merge({:align => "C", :style => "I"}))
      pdf.Ln

      pdf.Ln
    end

    if !(members.empty?)
      write_line(pdf, "#{l(:label_time_spent_by_member)}:", :style => "B")
      column_size = 190 / (to - from + 4)
      pdf.SetFillColor(230, 230, 230)
      pdf.SetFontStyle("", 9)
      column_options = {:width => column_size, :border => "LRTB"}
      text(pdf, l(:label_member), column_options.merge({:fill => 1}))
      text(pdf, l(:label_project), column_options.merge({:fill => 1}))
      (from .. to).each do |day|
        text(pdf, day.to_s, column_options.merge({:align => "C", :fill => 1}))
      end
      text(pdf, l(:label_total), column_options.merge({:align => "C", :fill => 1}))
      pdf.Ln

      pdf.SetFillColor(255, 255, 255)
      members.each do |member|
        if !(member[:projects].empty?)
          text(pdf, member[:member].name, column_options.merge({:width => column_size * 2, :style => "I"}))
          member[:totals].each do |total|
            text(pdf, (total > 0.0) ? ("%.2f" % total) : "", column_options.merge({:align => "C", :style => "I"}))
          end
          text(pdf, (member[:total] > 0.0) ? ("%.2f" % member[:total]) : "", column_options.merge({:align => "C", :style => "I"}))
          pdf.Ln

          member[:projects].each do |project|
            text(pdf, project[:project].name, column_options.merge({:width => column_size * 2, :align => "R", :style => ""}))
            project[:days].each do |day|
              text(pdf, (day > 0.0) ? ("%.2f" % day) : "", column_options.merge({:align => "C", :style => ""}))
            end
            text(pdf, (project[:total] > 0.0) ? ("%.2f" % project[:total]) : "", column_options.merge({:align => "C", :style => ""}))
            pdf.Ln
          end
        end
      end

      text(pdf, l(:label_total), column_options.merge({:width => column_size * 2, :style => "I"}))
      members_totals.each do |total|
        text(pdf, (total > 0.0) ? ("%.2f" % total) : "", column_options.merge({:align => "C", :style => "I"}))
      end
      text(pdf, (members_total > 0.0) ? ("%.2f" % members_total) : "", column_options.merge({:align => "C", :style => "I"}))
      pdf.Ln

      pdf.Ln
    end

    if show_released_versions
      write_line(pdf, "#{l(:label_versions_released)}:", :style => "B")
      column_size = 190 / (to - from + 2)
      pdf.SetFillColor(230, 230, 230)
      pdf.SetFontStyle("", 9)
      column_options = {:width => column_size, :border => "LRTB"}
      text(pdf, l(:label_project), column_options.merge({:fill => 1}))
      (from .. to).each do |day|
        text(pdf, day.to_s, column_options.merge({:align => "C", :fill => 1}))
      end
      pdf.Ln

      pdf.SetFillColor(255, 255, 255)
      projects.each do |project|
        if !(project[:versions].empty?)
          text(pdf, project[:project].name, column_options.merge({:align => "L", :style => ""}))
          project[:versions].each do |version|
            text(pdf, version.collect {|v| v.name}.join(", "), column_options.merge({:align => "C", :style => ""}))
          end
          pdf.Ln
        end
      end

      pdf.Ln
    end

    pdf.Output
  end
  
private

  def text(pdf, text, options = {})
    pdf.SetFontStyle(options[:style] || "", options[:font_size] || 9) if options[:style] or options[:font_size]
    pdf.Cell(options[:width] || 200,
             options[:height] || 5,
             text,
             options[:border] || 0,
             options[:line] || 0,
             options[:align] || "",
             options[:fill] || 0,
             options[:link] || "")
  end

  def write_line(pdf, line = "", options = {})
    text(pdf, line, options)
    pdf.Ln
  end
      
  def format_date(date)
    date.to_s
  end

end
